%
%  definition for the kml_colorstyle super (base) and sub (derived) class.
%
%  [kml]=kml_colorstyle(varargin)
%
%  where the optional varargin and defaults are:
%    id            (char, colorstyle id, '')
%    color         (char, opacity/color in hex aabbggrr, 'ffffffff')
%    colormode     (char, color mode , 'normal')
%
%  note that zero arguments constructs a default instance; one
%  argument of the class copies the instance; and two or more
%  arguments constructs a new instance from the arguments.
%
classdef kml_colorstyle < kml_substyle
    properties
        color     ='ffffffff';
        colormode ='normal';
    end
    
    methods
        function [kml]=kml_colorstyle(varargin)

            kml=kml@kml_substyle(varargin{:});

            switch nargin

%  create a default object

                case 0

%  copy the object or create the object from the input

                otherwise
                    if (nargin == 1) && isa(varargin{1},class(kml))
                        kml=varargin{1};

                    else
                        fnames=fieldnames(kml);

                        for i=length(fieldnames(kml_substyle()))+1:min(nargin,length(fnames))
                            if isa(varargin{i},class(kml.(fnames{i})))
                                if ~isempty(varargin{i})
                                    kml.(fnames{i})=varargin{i};
                                end
                            else
                                if ~isempty(inputname(i))
                                    warning('Argument ''%s'' for field ''%s'' is a ''%s'' class object, not ''%s''.',...
                                        inputname(i),fnames{i},class(varargin{i}),class(kml.(fnames{i})));
                                else
                                    warning('Argument %d for field ''%s'' is a ''%s'' class object, not ''%s''.',...
                                        i           ,fnames{i},class(varargin{i}),class(kml.(fnames{i})));
                                end
                            end
                        end
                    end

            end

        end

%  display the object

        function []=disp(kml)

            for i=1:numel(kml)
                if strcmp(class(kml),'kml_colorstyle')
                    disp(sprintf('class ''%s'' object ''%s%s'' = \n',...
                        class(kml),inputname(1),string_dim(kml,i)));
                end
                disp@kml_substyle(kml(i));
                disp(sprintf('         color: ''%s'''  ,kml(i).color));
                if strcmp(class(kml),'kml_colorstyle')
                    disp(sprintf('     colormode: ''%s''\n',kml(i).colormode));
                else
                    disp(sprintf('     colormode: ''%s'''  ,kml(i).colormode));
                end
            end

        end

%  write the object

        function []=kml_write(kml,fid,indent)

           if ~exist('fid','var') || isempty(fid)
               fid=1;
           end
           if ~exist('indent','var') || isempty(indent)
               indent='';
           end

%  loop over the colorstyles

            for i=1:numel(kml)
                if strcmp(class(kml),'kml_colorstyle')
                    if ~isempty(kml(i).id)
                        fprintf(fid,'%s<!ColorStyle id="%s">\n',indent,kml(i).id);
                    else
                        fprintf(fid,'%s<!ColorStyle>\n',indent);
                    end
                end
                kml_write@kml_substyle(kml(i),fid,indent);
                if ~isempty(kml(i).color)
                    fprintf(fid,'%s  <color>%s</color>\n',indent,kml(i).color);
                end
                if ~isempty(kml(i).colormode)
                    fprintf(fid,'%s  <colorMode>%s</colorMode>\n',indent,kml(i).colormode);
                end
                if strcmp(class(kml),'kml_colorstyle')
                    fprintf(fid,'%s</!ColorStyle>\n',indent);
                end
            end

        end
        
    end
    
end

